from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin

from .models import User


#
# Declare admin models
#

class UserAdmin(BaseUserAdmin):
    pass


#
# Register admin models
#

admin.site.register(User, UserAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-09 23:34
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('limitmonitor', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='limit',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='credit',
            name='external',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='limitmonitor.ExternalCredit'),
        ),
        migrations.AddField(
            model_name='credit',
            name='limit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='limitmonitor.Limit'),
        ),
    ]



from celery import shared_task

from .task_resources import common
from .task_resources import tunnel_credit
from .task_resources import tunnel_subscription


@shared_task
def tunnel_refresh_subscription():
    tunnel_subscription.monitor_woosub1_renewals()


@shared_task
def tunnel_new_subscription():
    tunnel_subscription.monitor_woosub1_new_subscriptions()


@shared_task
def tunnel_new_credit():
    tunnel_credit.monitor_woo1()


@shared_task
def deactivate_all_expired_limits():
    common.deactivate_all_expired_limits()


@shared_task
def debug_connection_task():
    common.debug_connection_task()



import ldap
import strictyaml
from decouple import Config, Csv, RepositoryIni
from django_auth_ldap.config import LDAPSearch

import purist.limitmonitor
from .settings_original import *

#
# LOAD CONFIGURATION FILE
#

CONFIG_PATH = '/etc/opt/purist/middleware/config.ini'
SECRET_PATH = '/etc/opt/purist/middleware/secret.ini'
LINK_PROFILE_PATH = '/etc/opt/purist/middleware/link_profile.strict.yml'

config = Config(RepositoryIni(CONFIG_PATH))
secret_config = Config(RepositoryIni(SECRET_PATH))

with open(LINK_PROFILE_PATH, 'r') as stream:
    LINK_PROFILE_ORDERED_DICT = strictyaml.load(stream.read()).data

#
# SECURITY
#

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = secret_config("DJANGO_SECRET_KEY")

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = config("DEBUG", cast=bool)

# it is safe to use these flags in production
DEBUG_ALL_ACCESS = config("DEBUG_ALL_ACCESS", cast=bool)
DEBUG_CHANGE_PASSWORD = config("DEBUG_CHANGE_PASSWORD", cast=bool)
DEBUG_SKIP_ACTIVATION_COMMAND = config("DEBUG_SKIP_ACTIVATION_COMMAND", cast=bool)
DEBUG_SKIP_VALIDATE_ON_AUTHENTICATION = config("DEBUG_SKIP_VALIDATE_ON_AUTHENTICATION", cast=bool)

# Required if DEBUG is False
ALLOWED_HOSTS = config("ALLOWED_HOSTS", cast=Csv())

#
# INSTALLED APPLICATIONS
#

INSTALLED_APPS += ["crispy_forms", "django_agpl", "django_celery_beat", "django_extensions", "ldapregister",
                   "limitmonitor", "purist"]

#
# AGPL APPLICATION
#

AGPL_ROOT = os.path.abspath(os.path.dirname(__file__) + "/..")

# no special exclusions are required, configuration and secrets are not stored in the site folder
AGPL_EXCLUDE_DIRS = [
    r'\.git$',
    r'\.idea$',
]

AGPL_FILENAME_PREFIX = 'middleware'

#
# REGISTRATION APPLICATION
#

REGISTRATION_OPEN = config("REGISTRATION_OPEN", cast=bool)

REG_PERSON_BASE_DN = config("REG_PERSON_BASE_DN")
REG_PERSON_OBJECT_CLASSES = config("REG_PERSON_OBJECT_CLASSES", cast=Csv())

REG_GROUP_BASE_DN = config("REG_GROUP_BASE_DN")
REG_GROUP_OBJECT_CLASSES = config("REG_GROUP_OBJECT_CLASSES", cast=Csv())

#
#  AUTHENTICATION
#

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'purist.custom.PassphraseValidator',
    },
]

AUTHENTICATION_BACKENDS = (
    'purist.custom.AuthenticationBackend',
)

AUTH_LDAP_SERVER_URI = config("AUTH_LDAP_SERVER_URI")
AUTH_LDAP_START_TLS = config("AUTH_LDAP_START_TLS", cast=bool)

AUTH_LDAP_BIND_DN = config("AUTH_LDAP_BIND_DN")
AUTH_LDAP_BIND_PASSWORD = secret_config("AUTH_LDAP_BIND_PASSWORD")

BASE_DN = config("AUTH_LDAP_USER_SEARCH_BASE_DN")
AUTH_LDAP_USER_SEARCH = LDAPSearch(BASE_DN, ldap.SCOPE_SUBTREE, "(uid=%(user)s)")
# must match `base_dn` and primary key in `ldapregister.models.LdapPerson`

AUTH_USER_MODEL = 'purist.User'

#
# DATABASE
#

# See also:
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases
# and https://pypi.python.org/pypi/django-ldapdb/
# (re-uses LDAP connection details from authentication settings)

SQLITE_DB_PATH = config("SQLITE_DB_PATH")

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': SQLITE_DB_PATH,
    },
    'ldap': {
        'ENGINE': 'ldapdb.backends.ldap',
        'NAME': AUTH_LDAP_SERVER_URI,
        'USER': AUTH_LDAP_BIND_DN,
        'PASSWORD': AUTH_LDAP_BIND_PASSWORD,
        'TLS': AUTH_LDAP_START_TLS,
    },
}

DATABASE_ROUTERS = ['ldapdb.router.Router']

#
# STATIC AND SITE SETTINGS
#

STATIC_ROOT = config("STATIC_ROOT")
STATICFILES_DIRS = config("STATICFILES_DIRS", cast=Csv())
SITE_TITLE = config("SITE_TITLE")
SITE_BYLINE = config("SITE_BYLINE")
SITE_DOMAIN = config("SITE_DOMAIN")
SITE_PROVIDER = config("SITE_PROVIDER")
SITE_PROVIDER_LINK = config("SITE_PROVIDER_LINK")

#
# WOOCOMMERCE
#

WOO_URL = config("WOO_URL")
WOO_WP_API = config("WOO_WP_API", cast=bool)
WOO_VERSION = config("WOO_VERSION")
WOO_QUERY_STRING_AUTH = config("WOO_QUERY_STRING_AUTH", cast=bool)  # required for OAuth over HTTPS

WOO_CONSUMER_KEY = secret_config("WOO_CONSUMER_KEY")
WOO_CONSUMER_SECRET = secret_config("WOO_CONSUMER_SECRET")

#
# WOOSUB1 PARSER
#

WOOSUB1_PRODUCT_LIST = config("WOOSUB1_PRODUCT_LIST", cast=Csv(int))

#
# SSH CONNECTION TO OPENVPN SERVER
#

OVPN_HOSTNAME = config("OVPN_HOSTNAME")
OVPN_PORT = config("OVPN_PORT", cast=int)
OVPN_USERNAME = config("OVPN_USERNAME")
OVPN_FILEPATH = config("OVPN_FILEPATH")

#
# LIMIT MONITOR
#

LM_SERVICES = purist.limitmonitor.ServicesContainer
LM_PARSERS = purist.limitmonitor.ParserContainer

#
# LOGGING
#

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'filters': None,
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}



from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import FileResponse
from django.shortcuts import render

from .models import Limit


@login_required
def userlimit(request):
    username = request.user.get_username()
    limits = Limit.objects.filter(user__username=username)

    # get flags for each limit
    has_limit = {}
    none_limit = True
    for limit in limits:
        label = limit.service_label().upper()
        has_limit[label] = limit.is_active

        if limit.is_active:
            none_limit = False

    has_limit["NONE"] = none_limit  # true if no limits are active

    render_data = {
        "DEBUG_CHANGE_PASSWORD": settings.DEBUG_CHANGE_PASSWORD,
        "username": username,
        "site_title": settings.SITE_TITLE,
        "site_byline": settings.SITE_BYLINE,
        "site_provider": settings.SITE_PROVIDER,
        "site_provider_link": settings.SITE_PROVIDER_LINK,
        "limits": limits,
        "has_limit": has_limit,
        "link_profile_ordered_dict": settings.LINK_PROFILE_ORDERED_DICT,
    }

    return render(request, 'limitmonitor/userlimit.html', render_data)


@login_required
def ovpn_userfile(request):
    user_identity = request.user.get_identity()
    filepath = settings.OVPN_FILEPATH.replace("{USER_IDENTITY}", user_identity)

    response = FileResponse(open(filepath, 'rb'), content_type='application/octet-stream')
    response['Content-Disposition'] = 'attachment; filename="purist.ovpn"'
    return response



# Copyright 2017-2018 Purism SPC
# SPDX-License-Identifier: AGPL-3.0-or-later

# Original file from Celery 4.0.2 documentation
# Copyright 2009-2016 Ask Solem
# http://docs.celeryproject.org/en/latest/django/first-steps-with-django.html
# SPDX-License-Identifier: CC-BY-SA-4.0

import os

from celery import Celery

# set the default Django settings module for the 'celery' program.
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'middleware.settings')

app = Celery('purist_middleware_monitor')

# Using a string here means the worker don't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object('django.conf:settings', namespace='CELERY')

# Load task modules from all registered Django app configs.
app.autodiscover_tasks()


@app.task(bind=True)
def debug_task(self):
    print('Request: {0!r}'.format(self.request))



"""middleware URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.11/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
from django.conf.urls import include, url
from django.contrib import admin
from django.views.generic import RedirectView
from registration.backends.simple.views import RegistrationView

import limitmonitor.views
import purist.views
from ldapregister.forms import RegistrationForm

#
# Set admin titles for this site
#

admin.site.site_title = "Site administration"
admin.site.site_header = "Site administration"

#
# Define patterns for this site
#

urlpatterns = [
    url(r'^$', purist.views.home, name='home'),
    url(r'^admin/', admin.site.urls),
    url(r'^accounts/$', RedirectView.as_view(url='/')),
    url(r'^accounts/profile/$', limitmonitor.views.userlimit, name='profile'),
    url(r'^accounts/profile/purist.ovpn', limitmonitor.views.ovpn_userfile, name='ovpn_userfile'),
    url(r'^accounts/register/$', RegistrationView.as_view(form_class=RegistrationForm), name='registration_register'),
    url(r'^accounts/', include('registration.backends.simple.urls')),
    url(r'^download/', include('django_agpl.urls')),
    url(r'^jslicense/$', purist.views.jslicense, name='jslicense'),
]



from choicesenum import ChoicesEnum
from django.conf import settings
from django.db import models
from django.utils import timezone


class Legacy(ChoicesEnum):
    UNDEFINED = 0, "Undefined"


def create_missing_user_limits(user):

    for code in settings.LM_SERVICES.MAP.keys():
        is_defined = code != settings.LM_SERVICES.UNDEFINED
        is_exists = Limit.objects.filter(user=user, service=code).exists()
        if is_defined and not is_exists:
            Limit(user=user, service=code).save()


def delete_unconverted_user_credits(user):
    identity = user.get_identity()
    ExternalCredit.objects.filter(account_name=identity, is_converted=False).delete()


class Limit(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    service = models.IntegerField(default=settings.LM_SERVICES.UNDEFINED,
                                  choices=settings.LM_SERVICES.choices())
    renewal_date = models.DateTimeField(default=None, blank=True, null=True)
    expiry_date = models.DateTimeField(default=None, blank=True, null=True)
    volume_total = models.DecimalField(default=0, decimal_places=2, max_digits=6)
    time_total = models.DecimalField(default=0, decimal_places=2, max_digits=6)
    is_active = models.BooleanField(default=False)
    created_date = models.DateTimeField(default=timezone.now)
    updated_date = models.DateTimeField(default=timezone.now)

    def service_label(self):

        return settings.LM_SERVICES.get_name_by_code(self.service)

    def active_label(self):

        if self.is_active:
            return "Yes"
        else:
            return "No"

    def endpoint_short_label(self):

        return self.endpoint_label("%Y-%m-%d")

    def endpoint_full_label(self):

        return self.endpoint_label("%Y-%m-%d %H:%M:%S %z")

    def endpoint_label(self, label_format):

        is_blank = self.renewal_date is None and self.expiry_date is None
        is_renewal = self.renewal_date is not None and self.expiry_date is None
        is_expiry = self.renewal_date is None and self.expiry_date is not None

        if is_blank:
            return "-"
        elif is_renewal:
            return self.renewal_date.strftime(label_format)
        elif is_expiry:
            return self.expiry_date.strftime(label_format)
        else:
            return "Invalid"

    def credit_label(self):

        if self.service == settings.LM_SERVICES.TUNNEL:
            return self.days_credit_label()
        else:
            return self.days_credit_label()

    def timedelta_to_daystring(self, delta):

        single_day = 60 * 60 * 24
        days = int(round(delta.total_seconds() / single_day))

        if days <= 62:
            return str(days) + " days"
        elif days <= 744:
            return str(days // 30) + " months"
        else:
            return str(days // 365) + " years"

    def days_credit_label(self):

        has_zero = self.expiry_date is None and self.renewal_date is None and self.time_total == 0
        has_credit_only = self.expiry_date is None and self.renewal_date is None and self.time_total > 0
        has_valid_expiry = self.expiry_date is not None and self.renewal_date is None and self.time_total == 0
        has_valid_renewal = self.expiry_date is None and self.renewal_date is not None and self.time_total == 0

        if has_zero:
            label = "-"
        elif has_credit_only:
            label = str(int(self.time_total)) + " days"
        elif has_valid_expiry:
            difference = self.expiry_date - timezone.now()
            label = self.timedelta_to_daystring(difference)
        elif has_valid_renewal:
            difference = self.renewal_date - timezone.now()
            label = self.timedelta_to_daystring(difference)
        else:
            label = "Invalid"

        return label


class ExternalBundle(models.Model):
    parser = models.IntegerField(default=settings.LM_PARSERS.UNDEFINED,
                                 choices=settings.LM_PARSERS.choices())
    external_key = models.CharField(max_length=30)
    service = models.IntegerField(default=settings.LM_SERVICES.UNDEFINED,
                                  choices=settings.LM_SERVICES.choices())
    time_credit = models.DecimalField(default=0, decimal_places=2, max_digits=6)
    volume_credit = models.DecimalField(default=0, decimal_places=2, max_digits=6)
    created_date = models.DateTimeField(default=timezone.now)
    updated_date = models.DateTimeField(default=timezone.now)


class ExternalCredit(models.Model):
    parser = models.IntegerField(default=settings.LM_PARSERS.UNDEFINED,
                                 choices=settings.LM_PARSERS.choices())
    external_key = models.CharField(max_length=30)
    label = models.CharField(max_length=30)
    bundle_key = models.CharField(max_length=30)
    bundle_label = models.CharField(max_length=30)
    quantity = models.DecimalField(default=1, decimal_places=2, max_digits=6)
    account_name = models.CharField(max_length=30, default="")
    additional_data = models.TextField(default="")
    error_message = models.TextField(default="")
    is_converted = models.BooleanField(default=False)
    created_date = models.DateTimeField(default=timezone.now)
    updated_date = models.DateTimeField(default=timezone.now)

    @property
    def parser_name(self):
        return settings.LM_PARSERS.get_name_by_code(self.parser)

    @property
    def external_code(self):
        return self.parser_name + ":" + str(self.external_key)

    @property
    def external_bundle(self):
        return self.parser_name + ":" + str(self.external_key)


class Credit(models.Model):
    limit = models.ForeignKey(Limit)
    external = models.ForeignKey(ExternalCredit, blank=True, null=True, default=None)
    time_credit = models.DecimalField(default=0, decimal_places=2, max_digits=6)
    volume_credit = models.DecimalField(default=0, decimal_places=2, max_digits=6)
    old_expiry_date = models.DateTimeField(blank=True, null=True, default=None)
    old_time_total = models.DecimalField(blank=True, null=True, default=None, decimal_places=2, max_digits=6)
    old_volume_total = models.DecimalField(blank=True, null=True, default=None, decimal_places=2, max_digits=6)
    new_expiry_date = models.DateTimeField(blank=True, null=True, default=None)
    new_time_total = models.DecimalField(blank=True, null=True, default=None, decimal_places=2, max_digits=6)
    new_volume_total = models.DecimalField(blank=True, null=True, default=None, decimal_places=2, max_digits=6)
    created_date = models.DateTimeField(default=timezone.now)
    updated_date = models.DateTimeField(default=timezone.now)



from .common import *


def get_latest_woo1(connection, product_id):
    return connection.get("orders?product=" + str(product_id))


def parse_woo1(json_entry, product_id):
    result_list = []
    order_name = json_entry["number"]
    order_id = json_entry["id"]

    for line_item in json_entry["line_items"]:
        if line_item["product_id"] == product_id:

            item_id = line_item["id"]
            product_id = line_item["product_id"]
            product_label = line_item["name"]
            quantity = line_item["quantity"]
            account = None  # FIXME: need to get the account
            external_key = str(order_id) + ":" + str(item_id)
            external_label = str(order_name)

            result_list.append({
                "parser": "WOO1",
                "external_key": external_key,
                "label": external_label,
                "product_key": product_id,
                "product_label": product_label,
                "quantity": quantity,
                "account": account,
                "original_email": "",
                "isconverted": False,
            })

    return result_list


def update_limit_woo1(ssh, credit):

    user, limit, external_bundle = get_limit_objects(credit)

    credit_days = int(external_bundle.time_credit * credit.quantity)
    credit_timedelta = timezone.timedelta(days=credit_days)
    activate_single_limit(ssh, limit, credit_timedelta, None)


def monitor_woo1():
    # make connection objects
    woo_connection = get_woo_connection()
    ssh = get_openvpn_ssh_connection()

    # get product sales and parse the results
    result_list = []
    for product_id in settings.WOO_PRODUCT_LIST:
        latest_woo1_json = get_latest_woo1(woo_connection, product_id).json()
        for json_entry in latest_woo1_json:
            try:
                result_list.extend(parse_woo1(json_entry, product_id))
            except Exception as e:
                logger.exception("Skipping JSON entry " + str(json_entry))

    # add new results
    count = 0
    for result in result_list:
        try:

            credit = ExternalCredit(
                parser=result["parser"],
                external_key=result["external_key"],
                label=result["label"],
                bundle_key=result["product_key"],
                bundle_label=result["product_label"],
                quantity=result["quantity"],
                account_name=result["account"],
                additional_data=result["original_email"],
                is_converted=False,
            )

            if not is_existing_credit(credit):
                store_credit_and_update_limit(ssh, credit)
                count += 1
            else:
                logger.debug("Skipped existing result " + str(result))
        except Exception as e:
            logger.exception("Skipped bad result " + str(result))

    if count > 0:
        logger.info("Added %i new results." % (count,))

    ssh.close()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-12-08 13:27
from __future__ import unicode_literals

import choicesenum.django.fields
from django.db import migrations

import limitmonitor.models


class Migration(migrations.Migration):
    dependencies = [
        ('limitmonitor', '0002_auto_20171009_2334'),
    ]

    operations = [
        migrations.AlterField(
            model_name='externalbundle',
            name='parser',
            field=choicesenum.django.fields.EnumIntegerField(default=limitmonitor.models.Legacy(0),
                                                             enum=limitmonitor.models.Legacy),
        ),
        migrations.AlterField(
            model_name='externalbundle',
            name='service',
            field=choicesenum.django.fields.EnumIntegerField(default=limitmonitor.models.Legacy(0),
                                                             enum=limitmonitor.models.Legacy),
        ),
        migrations.AlterField(
            model_name='externalcredit',
            name='parser',
            field=choicesenum.django.fields.EnumIntegerField(default=limitmonitor.models.Legacy(0),
                                                             enum=limitmonitor.models.Legacy),
        ),
        migrations.AlterField(
            model_name='limit',
            name='service',
            field=choicesenum.django.fields.EnumIntegerField(default=limitmonitor.models.Legacy(0),
                                                             enum=limitmonitor.models.Legacy),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-01-09 11:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('limitmonitor', '0003_auto_20171208_1327'),
    ]

    operations = [
        migrations.AlterField(
            model_name='externalbundle',
            name='service',
            field=models.IntegerField(choices=[(0, 'Undefined')], default=0),
        ),
        migrations.AlterField(
            model_name='limit',
            name='service',
            field=models.IntegerField(choices=[(0, 'Undefined')], default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-09 23:34
from __future__ import unicode_literals

from django.db import migrations, models
import ldapdb.models.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LdapGroup',
            fields=[
                ('dn', models.CharField(max_length=200, primary_key=False, serialize=False)),
                ('cn', ldapdb.models.fields.CharField(db_column='cn', max_length=200, primary_key=True, serialize=False)),
                ('description', ldapdb.models.fields.CharField(db_column='description', max_length=200)),
                ('members', ldapdb.models.fields.ListField(db_column='member')),
            ],
            options={
                'verbose_name': 'LDAP group',
                'verbose_name_plural': 'LDAP groups',
            },
        ),
        migrations.CreateModel(
            name='LdapPerson',
            fields=[
                ('dn', models.CharField(max_length=200, primary_key=False, serialize=False)),
                ('uid', ldapdb.models.fields.CharField(db_column='uid', max_length=200, primary_key=True, serialize=False)),
                ('cn', ldapdb.models.fields.CharField(db_column='cn', max_length=200)),
                ('description', ldapdb.models.fields.CharField(db_column='description', max_length=200)),
                ('sn', ldapdb.models.fields.CharField(db_column='sn', max_length=200)),
            ],
            options={
                'verbose_name': 'LDAP person',
                'verbose_name_plural': 'LDAP people',
            },
        ),
    ]



from django.conf import settings
from django.shortcuts import render


def home(request):
    render_data = {
        "username": request.user.get_username(),
        "site_title": settings.SITE_TITLE,
        "site_byline": settings.SITE_BYLINE,
        "site_provider": settings.SITE_PROVIDER,
        "site_provider_link": settings.SITE_PROVIDER_LINK,
    }

    return render(request, 'purist/home.html', render_data)


def jslicense(request):
    render_data = {
        "site_title": settings.SITE_TITLE,
    }

    return render(request, 'purist/jslicense.html', render_data)



from django.contrib import admin

from .models import Credit, ExternalBundle, ExternalCredit, Limit


#
# Declare admin models
#

class CreditAdmin(admin.ModelAdmin):
    list_display = ['id', 'limit', 'external', 'time_credit', 'volume_credit']


class ExternalBundleAdmin(admin.ModelAdmin):
    list_display = ['id', 'parser', 'external_key', 'service', 'time_credit', 'volume_credit']


class ExternalCreditAdmin(admin.ModelAdmin):
    list_display = ['label', 'external_code', 'bundle_key', 'account_name', 'is_converted', 'error_message']


class LimitAdmin(admin.ModelAdmin):
    list_display = ['id', 'user', 'service', 'is_active', 'renewal_date', 'expiry_date', 'volume_total', 'time_total']
    list_filter = ['service', 'renewal_date', 'user']

#
# Register admin models
#

admin.site.register(Credit, CreditAdmin)
admin.site.register(ExternalBundle, ExternalBundleAdmin)
admin.site.register(ExternalCredit, ExternalCreditAdmin)
admin.site.register(Limit, LimitAdmin)



from django.contrib import admin

from .models import LdapGroup, LdapPerson


#
# Declare admin models
#

class LdapGroupAdmin(admin.ModelAdmin):
    exclude = ['dn', 'objectClass']
    list_display = ['cn', 'description', ]


class LdapPersonAdmin(admin.ModelAdmin):
    exclude = ['dn', 'objectClass']
    list_display = ['uid', 'mail', 'description', ]


#
# Register admin models
#

admin.site.register(LdapGroup, LdapGroupAdmin)
admin.site.register(LdapPerson, LdapPersonAdmin)



import datetime

from .common import *


def parse_woosub1(json_entry):

    subscription_id = str(json_entry["id"])

    # validation

    if len(json_entry["line_items"]) != 1:
        raise Exception("Too many line items in subscription %s" % (id, ))

    line_item = json_entry["line_items"][0]

    quantity = line_item["quantity"]
    if quantity != 1:
        raise Exception("Bad quantity %s in subscription %s" % (quantity, id, ))

    # calculate next renewal date

    next_renewal_naive = datetime.datetime.strptime(json_entry["next_payment_date"], "%Y-%m-%dT%H:%M:%S")
    next_renewal = timezone.make_aware(next_renewal_naive)

    # get account name
    username = get_username_from_woo_customer_id(json_entry["customer_id"])

    # create result

    result = {
        "parser": 2,  # FIXME: this is WOO_SUBSCRIPTION_V1 from purist.limitmonitor
        "external_key": subscription_id,
        "label": json_entry["number"],
        "bundle_key": str(line_item["product_id"]),
        "bundle_label": str(line_item["name"]),
        "quantity": 1,
        "account": username,
        "next_renewal": next_renewal,
    }

    return [result, ]


def monitor_woosub1_new_subscriptions():

    # make connection objects
    woo_connection = get_woo_connection()
    ssh = get_openvpn_ssh_connection()

    # initialise
    result_list = []
    latest_subscription_json = woo_connection.get("subscriptions?orderby=date&order=desc").json()

    # parse recent subscriptions and store results
    for json_entry in latest_subscription_json:
        try:
            product_id = int(json_entry["line_items"][0]["product_id"])
            if product_id in settings.WOOSUB1_PRODUCT_LIST:
                result_list.extend(parse_woosub1(json_entry))
        except Exception as e:
            logger.exception("Skipping JSON entry " + str(json_entry))

    # add new results
    count = 0
    for result in result_list:
        try:

            credit = ExternalCredit(
                parser=result["parser"],
                external_key=result["external_key"],
                label=result["label"],
                bundle_key=result["bundle_key"],
                bundle_label=result["bundle_label"],
                quantity=result["quantity"],
                account_name=result["account"],
                additional_data="None",
                is_converted=False,
            )

            if not is_existing_credit(credit):
                store_credit_and_update_limit(ssh, credit, result["next_renewal"])
                count += 1
            else:
                logger.debug("Skipped existing result " + str(result))
        except Exception as e:
            logger.exception("Skipped bad result " + str(result))

    ssh.close()


def update_limit_woosub1(ssh, credit, renewal_date):

    user, limit, external_bundle = get_limit_objects(credit)
    activate_single_limit(ssh, limit, None, renewal_date)


def monitor_woosub1_renewals():

    # make connection objects
    woo_connection = get_woo_connection()
    ssh = get_openvpn_ssh_connection()

    # get objects due for renewal (one hour grace / buffer)
    now = timezone.now() - datetime.timedelta(hours=1)
    overdue_list = Limit.objects.filter(renewal_date__lte=now,expiry_date=None, is_active=True)

    count = 0
    for limit in overdue_list:

        identity = limit.user.get_identity()
        woosub_bundle_list = ExternalBundle.objects.filter(service=limit.service, parser="WOOSUB1")

        for bundle in woosub_bundle_list:

            woosub_credit = ExternalCredit.objects.get(account_name=identity, bundle_key=bundle.external_key,
                                                       parser="WOOSUB1")

            try:
                subscription_query = "subscriptions/" + woosub_credit.external_key
                woosub_json = woo_connection.get(subscription_query).json()
                result = parse_woosub1(woosub_json)

                # deactivate expired limits
                if not result["active"]:
                    deactivate_single_limit(ssh, limit)
                    woosub_credit.is_converted = False
                    woosub_credit.error_message = "Expired."
                else:
                    store_credit_and_update_limit(ssh, woosub_credit, result["renewal"])
                    woosub_credit.is_converted = True
                    woosub_credit.error_message = ""

                count += 1

            except Exception as e:
                message = "Skipping bad credit %s (%s)." % (woosub_credit.id, woosub_credit.external_code)
                logger.exception(message)
                woosub_credit.is_converted = False
                woosub_credit.error_message = message + repr(e)
            finally:
                woosub_credit.save()

    if count > 0:
        logger.info("Updated %i subscriptions." % (count,))

    ssh.close()



from django.contrib.auth import get_user_model
from django.contrib.auth.forms import AuthenticationForm as BaseAuthenticationForm
from django.utils.translation import ugettext_lazy as _
from registration.forms import RegistrationForm as BaseRegistrationForm

User = get_user_model()


class AuthenticationForm(BaseAuthenticationForm):
    # this currently has no effect

    def __init__(self, request=None, *args, **kwargs):
        super(AuthenticationForm, self).__init__(request, *args, **kwargs)
        self.fields["password"].label = _("Passphrase")


class RegistrationForm(BaseRegistrationForm):

    email = None  # override base definition to remove email field

    def __init__(self, *args, **kwargs):
        super(RegistrationForm, self).__init__(*args, **kwargs)
        self.fields[User.USERNAME_FIELD].label = _("Username")
        self.fields["password1"].label = _("Passphrase")
        self.fields["password2"].label = _("Passphrase confirmation")
        self.fields["password2"].help_text = _("Enter the same passphrase as before, for verification.")

    class Meta(BaseRegistrationForm.Meta):

        model = User

        # override base definition to remove email field
        fields = (
            User.USERNAME_FIELD,
            'password1',
            'password2',
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-02-15 11:33
from __future__ import unicode_literals

import ldapdb.models.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('ldapregister', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='ldapperson',
            name='mail',
            # field=ldapdb.models.fields.CharField(db_column='mail', default='none', max_length=200),
            field=ldapdb.models.fields.CharField(db_column='mail', max_length=200),
            preserve_default=False,
        ),
    ]



import logging

import ldap
import ldapdb.models
import pyasn1.codec.ber.encoder
import pyasn1.type.namedtype
import pyasn1.type.univ
from django.conf import settings
from django.db import connections, router
from ldapdb.models.fields import CharField, ListField

log = logging.getLogger(__name__)


class LdapGroup(ldapdb.models.Model):
    """
    Class for representing an LDAP group entry.
    """

    class Meta:
        verbose_name = "LDAP group"
        verbose_name_plural = "LDAP groups"

    # LDAP meta-data
    base_dn = settings.REG_GROUP_BASE_DN
    object_classes = settings.REG_GROUP_OBJECT_CLASSES

    # LDAP group attributes
    cn = CharField(db_column='cn', max_length=200, primary_key=True)
    description = CharField(db_column='description', max_length=200)
    members = ListField(db_column='member')

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name


class LdapPerson(ldapdb.models.Model):
    """
    Class for representing an LDAP person entry.
    """

    class Meta:
        verbose_name = "LDAP person"
        verbose_name_plural = "LDAP people"

    # LDAP meta-data
    base_dn = settings.REG_PERSON_BASE_DN
    object_classes = settings.REG_PERSON_OBJECT_CLASSES

    # Minimal attributes
    uid = CharField(db_column='uid', max_length=200, primary_key=True)
    cn = CharField(db_column='cn', max_length=200)
    description = CharField(db_column='description', max_length=200)
    sn = CharField(db_column='sn', max_length=200)
    mail = CharField(db_column='mail', max_length=200)

    def __str__(self):
        return self.uid

    def __unicode__(self):
        return self.uid

    def change_password(self, raw_password, using=None):
        # dig into the ldapdb primitives
        using = using or router.db_for_write(self.__class__, instance=self)
        connection = connections[using]
        cursor = connection._cursor()

        # call pyldap_orm password modification
        cursor.connection.extop_s(PasswordModify(self.dn, raw_password))


# The following code taken from https://github.com/asyd/pyldap_orm/blob/master/pyldap_orm/controls.py
# Copyright 2016 Bruno Bonfils
# SPDX-License-Identifier: Apache-2.0 (no NOTICE file)


class PasswordModify(ldap.extop.ExtendedRequest):
    """
    Implements RFC 3062, LDAP Password Modify Extended Operation
    Reference: https://www.ietf.org/rfc/rfc3062.txt
    """

    def __init__(self, identity, new, current=None):
        self.requestName = '1.3.6.1.4.1.4203.1.11.1'
        self.identity = identity
        self.new = new
        self.current = current

    def encodedRequestValue(self):
        request = self.PasswdModifyRequestValue()
        request.setComponentByName('userIdentity', self.identity)
        if self.current is not None:
            request.setComponentByName('oldPasswd', self.current)
        request.setComponentByName('newPasswd', self.new)
        return pyasn1.codec.ber.encoder.encode(request)

    class PasswdModifyRequestValue(pyasn1.type.univ.Sequence):
        """
        PyASN1 representation of:
            PasswdModifyRequestValue ::= SEQUENCE {
            userIdentity    [0]  OCTET STRING OPTIONAL
            oldPasswd       [1]  OCTET STRING OPTIONAL
            newPasswd       [2]  OCTET STRING OPTIONAL }
        """
        componentType = pyasn1.type.namedtype.NamedTypes(
            pyasn1.type.namedtype.OptionalNamedType(
                'userIdentity',
                pyasn1.type.univ.OctetString().subtype(
                    implicitTag=pyasn1.type.tag.Tag(pyasn1.type.tag.tagClassContext, pyasn1.type.tag.tagFormatSimple, 0)
                )),
            pyasn1.type.namedtype.OptionalNamedType(
                'oldPasswd',
                pyasn1.type.univ.OctetString().subtype(
                    implicitTag=pyasn1.type.tag.Tag(pyasn1.type.tag.tagClassContext, pyasn1.type.tag.tagFormatSimple, 1)
                )),
            pyasn1.type.namedtype.OptionalNamedType(
                'newPasswd',
                pyasn1.type.univ.OctetString().subtype(
                    implicitTag=pyasn1.type.tag.Tag(pyasn1.type.tag.tagClassContext, pyasn1.type.tag.tagFormatSimple, 2)
                )),
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-12-08 13:27
from __future__ import unicode_literals

from django.db import migrations, models

import purist.custom


class Migration(migrations.Migration):
    dependencies = [
        ('purist', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='username',
            field=models.CharField(error_messages={'unique': 'A user with that username already exists.'},
                                   help_text='Required. Start with a letter, followed by letters and numbers.',
                                   max_length=150, unique=True, validators=[purist.custom.UsernameValidator()],
                                   verbose_name='username'),
        ),
    ]



import pathlib

import django.contrib.auth
import paramiko
from celery.utils.log import get_task_logger
from django.conf import settings
from django.db import transaction
from django.utils import timezone
from woocommerce import API as WOO_API

from ..models import ExternalCredit, ExternalBundle, Limit

logger = get_task_logger(__name__)


def get_woo_connection():
    return WOO_API(
        url=settings.WOO_URL,
        consumer_key=settings.WOO_CONSUMER_KEY,
        consumer_secret=settings.WOO_CONSUMER_SECRET,
        wp_api=settings.WOO_WP_API,
        version=settings.WOO_VERSION,
        query_string_auth=settings.WOO_QUERY_STRING_AUTH,
    )


def get_username_from_woo_customer_id(customer_id, woo=None):
    if woo is None:
        woo = get_woo_connection()

    try:
        query = "customers/" + str(customer_id)
        result = woo.get(query).json()
        return result["username"] + "@" + settings.SITE_DOMAIN
    except Exception as e:
        logger.exception("Could not retrieve username for customer_id " + str(customer_id))
        return "invalid"

    return account


def get_openvpn_ssh_connection():
    # make ssh connection to OpenVPN server
    # (uses system host keys, warns if host is not recognised)
    ssh = paramiko.SSHClient()
    ssh.load_system_host_keys()
    ssh.set_missing_host_key_policy(paramiko.WarningPolicy())  # TODO: where is this logged?
    ssh.connect(
        hostname=settings.OVPN_HOSTNAME,
        port=settings.OVPN_PORT,
        username=settings.OVPN_USERNAME,
    )

    return ssh


def managed_exec(ssh, command, close=True):
    stdin, stdout, stderr = ssh.exec_command(command)

    if close:
        ssh.close()

    output = "".join(stdout.readlines()).strip()
    if output == "":
        output = "None."

    error = "".join(stderr.readlines()).strip()

    message = "Executed: %s Output: %s" % (command, output,)

    # on sucess, log output, otherwise raise exception
    if stdout.channel.recv_exit_status() == 0:
        logger.info(message)
    else:
        message += " Error: " + error
        raise Exception(message)


def is_existing_credit(credit):
    matching_credits = ExternalCredit.objects.filter(
        parser=credit.parser,
        external_key=credit.external_key,
    )

    is_existing = len(matching_credits) > 0

    return is_existing


def get_external_bundle(parser, external_key):
    return ExternalBundle.objects.get(
        parser=parser,
        external_key=external_key,
    )


def activate_single_limit(ssh, limit, credit_timedelta=None, renewal_date=None):

    is_credit = credit_timedelta is not None
    is_renewal = renewal_date is not None

    if is_credit == is_renewal:
        raise Exception("Invalid activation attempt. Need strictly one of credit or renewal data.")

    if is_credit and limit.is_active:
        limit.expiry_date += credit_timedelta
    elif is_credit and not limit.is_active:
        limit.expiry_date = timezone.now() + credit_timedelta
        limit.is_active = True
    elif is_renewal:
        limit.renewal_date = renewal_date
        limit.is_active = True
    else:
        raise Exception("Invalid activation attempt. Unknown condition.")

    # skip activation command if we are debugging
    if settings.DEBUG_SKIP_ACTIVATION_COMMAND:
        limit.save()
        return

    # otherwise, activate the limit before saving

    if limit.service == settings.LM_SERVICES.TUNNEL:
        user_identity = limit.user.get_identity()
        filepath = settings.OVPN_FILEPATH.replace("{USER_IDENTITY}", user_identity)
        is_file = pathlib.Path(filepath).is_file()

        # only create certificate if it doesn't exist
        if not is_file:
            command = "./create_new_ovpn_config --generate %s" % (user_identity,)
            managed_exec(ssh, command, close=False)
    else:
        # skip unsupported limits
        limit.is_active = False

    limit.save()


def deactivate_single_limit(ssh, limit):
    if limit.service == settings.LM_SERVICES.TUNNEL:
        command = "./create_new_ovpn_config --revoke %s" % (limit.user.get_identity(),)
        managed_exec(ssh, command, close=False)

    limit.is_active = False
    limit.save()


def get_limit_objects(credit):

    # get and validate local username

    suffix = "@" + settings.SITE_DOMAIN
    if credit.account_name is None or not str.endswith(credit.account_name, suffix):
        raise Exception("Invalid account name: " + str(credit.account_name))
    else:
        suffix_len = 0 - len(suffix)
        username = credit.account_name[:suffix_len]

    # get objects (implicit validation that they exist)

    external_bundle = get_external_bundle(credit.parser, credit.bundle_key)

    limit = Limit.objects.get(
        user__username=username,
        service=external_bundle.service,
    )

    user = django.contrib.auth.get_user_model().objects.get(
        username=username,
    )

    return user, limit, external_bundle

@transaction.atomic
def store_credit_and_update_limit(ssh, credit, next_renewal=None):
    try:
        if credit.parser == "WOO1":
            from .tunnel_credit import update_limit_woo1
            update_limit_woo1(ssh, credit)
        elif credit.parser == 2:  # FIXME: this is WOO_SUBSCRIPTION_V1 from purist.limitmonitor
            from .tunnel_subscription import update_limit_woosub1
            update_limit_woosub1(ssh, credit, next_renewal)
        else:
            raise Exception("Unrecognised Parser " + str(credit.parser))

        credit.is_converted = True
        credit.error_message = ""
    except Exception as e:
        message = "Skipped adding credit Parser " + str(credit.parser) + ": " + credit.external_key + ". "
        logger.exception(message)
        credit.error_message = message + repr(e)
    finally:
        credit.save()
        state = "converted" if credit.is_converted else "skipped"
        logger.info("Stored " + state + " credit Parser " + str(credit.parser) + ": " + credit.external_key)


def deactivate_all_expired_limits():
    # make connection objects
    ssh = get_openvpn_ssh_connection()

    # get overdue objects and deactivate them
    now = timezone.now()
    overdue_list = Limit.objects.filter(expiry_date__lte=now, is_active=True)

    for limit in overdue_list:
        deactivate_single_limit(ssh, limit)

    ssh.close()


def debug_connection_task():
    # make connection objects
    woo_connection = get_woo_connection()
    ssh = get_openvpn_ssh_connection()
    managed_exec(ssh, "whoami", close=True)

    logger.info("Debug task with " + repr(woo_connection) + " and " + repr(ssh) + " completed successfully.")



from collections import OrderedDict

from django.utils.translation import ugettext_lazy as _


class Container:
    UNDEFINED = 0

    MAP = OrderedDict([
        (UNDEFINED, _("Undefined"))
    ])

    @classmethod
    def choices(cls):
        choices = []
        for code, name in cls.MAP.items():
            choices.append((code, name))

        return choices

    @classmethod
    def get_name_by_code(cls, code):
        return cls.MAP.get(code, cls.MAP.get(cls.UNDEFINED))


class ServicesContainer(Container):
    # CONSTANTS
    UNDEFINED = 0
    TUNNEL = 1
    COMMUNICATION = 2

    # MAPPING
    MAP = OrderedDict([
        (UNDEFINED, _("Undefined")),
        (TUNNEL, _("Tunnel")),
        (COMMUNICATION, _("Communication")),
    ])


class ParserContainer(Container):
    # CONSTANTS
    UNDEFINED = 0
    WOO_PRODUCT_V1 = 1
    WOO_SUBSCRIPTION_V1 = 2

    # MAPPING
    MAP = OrderedDict([
        (UNDEFINED, _("Undefined")),
        (WOO_PRODUCT_V1, _("WooCommerce Product v1")),
        (WOO_SUBSCRIPTION_V1, _("WooCommerce Subscription v1")),
    ])



import logging

from django.conf import settings
from django.contrib.auth.models import AbstractUser
from django.contrib.auth.models import UserManager as BaseUserManager
from django.core import validators
from django.db import models
from django.utils import timezone
from django.utils.crypto import salted_hmac
from django.utils.deconstruct import deconstructible
from django.utils.translation import ugettext_lazy as _

from ldapregister.models import LdapPerson
from limitmonitor import models as limitmonitor_models
from limitmonitor.task_resources import common as limitmonitor_common

log = logging.getLogger(__name__)


@deconstructible
class UsernameValidator(validators.RegexValidator):
    regex = r'^[A-Za-z][A-Za-z0-9]*$'
    message = _(
        'Enter a valid username. Must start with a letter, followed by letters and numbers.'
        ' No punctuation or special characters.'
    )


class UserManager(BaseUserManager):

    def create_user(self, username, email=None, password=None, **extra_fields):
        """Create regular users in LDAP, with no Django password."""

        return super(UserManager, self).create_user(username, email, password, **extra_fields)

    def create_superuser(self, username, email=None, password=None, **extra_fields):
        """Create superusers with a Django password."""

        return super(UserManager, self).create_superuser(username, email, password, **extra_fields)


class User(AbstractUser):
    objects = UserManager()
    REQUIRED_FIELDS = []
    username_validator = UsernameValidator()

    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=_('Required. Start with a letter, followed by letters and numbers.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )

    @classmethod
    def normalize_username(cls, username):
        username = super(User, cls).normalize_username(username)

        username = username.lower()
        suffix = "@" + settings.SITE_DOMAIN.lower()
        offset = 0 - len(suffix)

        if username.endswith(suffix):
            username = username[:offset]

        return username

    def get_ldap(self):
        return LdapPerson.objects.get(uid=self.get_username())

    def has_ldap(self):
        result = LdapPerson.objects.filter(uid=self.get_username())
        return len(result) == 1

    def create_ldap(self):
        username = self.get_username()
        mail = self.get_identity()
        LdapPerson.objects.create(uid=username, cn=username, sn=username, mail=mail)

    def set_ldap_password(self, raw_password):
        ldap_person = self.get_ldap()
        ldap_person.change_password(raw_password)

    def get_identity(self):
        return self.get_username() + "@" + settings.SITE_DOMAIN.lower()

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):

        # save django user
        super(User, self).save(force_insert, force_update, using, update_fields)

        # create LDAP user (if required)
        if not self.has_ldap():
            self.create_ldap()

        # force null Django password (will use LDAP password instead)
        self.set_unusable_password()

        # create any missing limits
        limitmonitor_models.create_missing_user_limits(self)

        # delete invalid credits (they will be re-parsed)
        limitmonitor_models.delete_unconverted_user_credits(self)

        if settings.DEBUG_ALL_ACCESS:

            ssh = limitmonitor_common.get_openvpn_ssh_connection()
            renewal_date = timezone.now() + timezone.timedelta(weeks=5200)

            for limit in limitmonitor_models.Limit.objects.filter(user=self, is_active=False):
                limitmonitor_common.activate_single_limit(ssh, limit, renewal_date=renewal_date)

            ssh.close()

    def set_password(self, raw_password):

        # force null Django password (will use LDAP password)
        self.set_unusable_password()

        # create LDAP user (if required)
        if self.get_username():

            if not self.has_ldap():
                self.create_ldap()

            # set LDAP password
            self.set_ldap_password(raw_password)

    def get_session_auth_hash(self):
        """
        Return an HMAC of the password field.
        """
        key_salt = "django.contrib.auth.models.AbstractBaseUser.get_session_auth_hash"
        return salted_hmac(key_salt, self.get_username()).hexdigest()  # FIXME: should use LDAP password value!



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-09 23:34
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Credit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_credit', models.DecimalField(decimal_places=2, default=0, max_digits=6)),
                ('volume_credit', models.DecimalField(decimal_places=2, default=0, max_digits=6)),
                ('old_expiry_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('old_time_total', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=6, null=True)),
                ('old_volume_total', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=6, null=True)),
                ('new_expiry_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('new_time_total', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=6, null=True)),
                ('new_volume_total', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=6, null=True)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_date', models.DateTimeField(default=django.utils.timezone.now)),
            ],
        ),
        migrations.CreateModel(
            name='ExternalBundle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('parser', models.CharField(choices=[('WOO1', 'WooCommerce v1'), ('WOOSUB1', 'WooCommerce Subscription v1')], max_length=30)),
                ('external_key', models.CharField(max_length=30)),
                ('service', models.CharField(choices=[('TUNNEL', 'Tunnel'), ('COMMUNICATION', 'Communication')], max_length=30)),
                ('time_credit', models.DecimalField(decimal_places=2, default=0, max_digits=6)),
                ('volume_credit', models.DecimalField(decimal_places=2, default=0, max_digits=6)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_date', models.DateTimeField(default=django.utils.timezone.now)),
            ],
        ),
        migrations.CreateModel(
            name='ExternalCredit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('parser', models.CharField(choices=[('WOO1', 'WooCommerce v1'), ('WOOSUB1', 'WooCommerce Subscription v1')], max_length=30)),
                ('external_key', models.CharField(max_length=30)),
                ('label', models.CharField(max_length=30)),
                ('bundle_key', models.CharField(max_length=30)),
                ('bundle_label', models.CharField(max_length=30)),
                ('quantity', models.DecimalField(decimal_places=2, default=1, max_digits=6)),
                ('account_name', models.CharField(default='', max_length=30)),
                ('additional_data', models.TextField(default='')),
                ('error_message', models.TextField(default='')),
                ('is_converted', models.BooleanField(default=False)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_date', models.DateTimeField(default=django.utils.timezone.now)),
            ],
        ),
        migrations.CreateModel(
            name='Limit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('service', models.CharField(choices=[('TUNNEL', 'Tunnel'), ('COMMUNICATION', 'Communication')], max_length=30)),
                ('renewal_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('expiry_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('volume_total', models.DecimalField(decimal_places=2, default=0, max_digits=6)),
                ('time_total', models.DecimalField(decimal_places=2, default=0, max_digits=6)),
                ('is_active', models.BooleanField(default=False)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_date', models.DateTimeField(default=django.utils.timezone.now)),
            ],
        ),
    ]



import logging

from django.conf import settings
from django.contrib.auth.password_validation import MinimumLengthValidator as BaseValidator
from django.utils.translation import ugettext_lazy as _
from django_auth_ldap.backend import LDAPBackend as BaseBackend
from woocommerce import API as WOOCOMMERCE_API

from .models import User, UsernameValidator

log = logging.getLogger(__name__)


class AuthenticationBackend(BaseBackend):

    def __init__(self, *args, **kwargs):
        super(AuthenticationBackend, self).__init__(*args, **kwargs)

    def is_valid_jwt(self, username=None, password=None):

        is_valid = False

        try:

            jwt_wcapi = WOOCOMMERCE_API(
                url=settings.WOO_URL,
                consumer_key=settings.WOO_CONSUMER_KEY,
                consumer_secret=settings.WOO_CONSUMER_SECRET,
                wp_api=True,
                version="jwt-auth/v1",  # required for JWT Authentication
                query_string_auth=settings.WOO_QUERY_STRING_AUTH,
            )

            jwt_response = jwt_wcapi.post("token", {"username": username, "password": password})

            jwt_status = jwt_response.status_code
            jwt_token = jwt_response.json().get("token", None)
            jwt_code = jwt_response.json().get("code", None)

            known_codes = ["[jwt_auth] incorrect_password", "[jwt_auth] invalid_username"]

            if jwt_status == 200 and jwt_token is not None:
                is_valid = True
            elif jwt_status == 403 and jwt_code in known_codes:
                # recognised authentication failure
                is_valid = False
            else:
                # raise exception for an unrecognised failure
                raise Exception("Unrecognised JWT response: %s" % (jwt_response.json(),))

        except Exception as e:
            logging.exception("JWT authentication failed with an unrecognised error: %s" % (e,))
        finally:
            return is_valid

    def authenticate(self, request=None, username=None, password=None, **kwargs):
        user_model = User
        normalized_username = user_model.normalize_username(username)

        # first, validate username (even if it exists, username must be valid)

        if not settings.DEBUG_SKIP_VALIDATE_ON_AUTHENTICATION:
            validator = UsernameValidator()
            validator(username)

        # second, attempt LDAP authentication (with early exit on success)

        user = super(AuthenticationBackend, self).authenticate(request, normalized_username, password, **kwargs)

        if user is not None:
            return user

        # third, attempt WooCommerce/JWT authentication
        # (if successful, create and return LDAP user, otherwise return None)

        if self.is_valid_jwt(normalized_username, password):

            # try to get a preexisting user object
            # otherwise create a new one

            try:
                user = user_model.objects.get(username=username)
            except user_model.DoesNotExist:
                user = user_model(username=username, email=None)

            # update/set user details
            user.email = user.get_identity()
            user.set_password(password)
            user.save()

            return super(AuthenticationBackend, self).authenticate(request, normalized_username, password, **kwargs)
        else:
            return None


class PassphraseValidator(BaseValidator):
    # TODO: bundle in all the other validators from django.contrib.auth.password_validation

    def __init__(self, min_length=15, *args, **kwargs):
        super(PassphraseValidator, self).__init__(min_length, *args, **kwargs)

    def get_help_text(self):
        return _("A good passphrase is made of at least three long words.")
